@@ -1,13 +1,16 @@
-#
-# This example demonstrates usage of the Indenter class.
-#
-# Since indentation is context-sensitive, a postlex stage is introduced to
-# manufacture INDENT/DEDENT tokens.
-#
-# It is crucial for the indenter that the NL_type matches
-# the spaces (and tabs) after the newline.
-#
+"""
+Parsing Indentation
+===================
 
+A demonstration of parsing indentation (“whitespace significant” language)
+and the usage of the Indenter class.
+
+Since indentation is context-sensitive, a postlex stage is introduced to
+manufacture INDENT/DEDENT tokens.
+
+It is crucial for the indenter that the NL_type matches
+the spaces (and tabs) after the newline.
+"""
 from lark import Lark
 from lark.indenter import Indenter
 
