@@ -1,11 +1,9 @@
-from __future__ import print_function
-
 ###{standalone
 #
 #
 #   Lark Stand-alone Generator Tool
 # ----------------------------------
-# Generates a stand-alone LALR(1) parser with a standard lexer
+# Generates a stand-alone LALR(1) parser
 #
 # Git:    https://github.com/erezsh/lark
 # Author: Erez Shinan (erezshin@gmail.com)
@@ -26,25 +24,29 @@ from __future__ import print_function
 #
 #
 
-import os
-from io import open
+from abc import ABC, abstractmethod
+from collections.abc import Sequence
+from types import ModuleType
+from typing import (
+    TypeVar, Generic, Type, Tuple, List, Dict, Iterator, Collection, Callable, Optional, FrozenSet, Any,
+    Union, Iterable, IO, TYPE_CHECKING,
+    Pattern as REPattern, ClassVar, Set, Mapping
+)
 ###}
 
-import codecs
 import sys
 import token, tokenize
 import os
-from pprint import pprint
 from os import path
 from collections import defaultdict
 from functools import partial
+from argparse import ArgumentParser
 
 import lark
-from lark import Lark
-from lark.parsers.lalr_analysis import Reduce
+from lark.tools import lalr_argparser, build_lalr, make_warnings_comments
 
 
-from lark.grammar import RuleOptions, Rule
+from lark.grammar import Rule
 from lark.lexer import TerminalDef
 
 _dir = path.dirname(__file__)
@@ -57,7 +59,6 @@ EXTRACT_STANDALONE_FILES = [
     'utils.py',
     'tree.py',
     'visitors.py',
-    'indenter.py',
     'grammar.py',
     'lexer.py',
     'common.py',
@@ -66,26 +67,27 @@ EXTRACT_STANDALONE_FILES = [
     'parsers/lalr_analysis.py',
     'parser_frontends.py',
     'lark.py',
+    'indenter.py',
 ]
 
 def extract_sections(lines):
     section = None
     text = []
     sections = defaultdict(list)
-    for l in lines:
-        if l.startswith('###'):
-            if l[3] == '{':
-                section = l[4:].strip()
-            elif l[3] == '}':
+    for line in lines:
+        if line.startswith('###'):
+            if line[3] == '{':
+                section = line[4:].strip()
+            elif line[3] == '}':
                 sections[section] += text
                 section = None
                 text = []
             else:
-                raise ValueError(l)
+                raise ValueError(line)
         elif section:
-            text.append(l)
+            text.append(line)
 
-    return {name:''.join(text) for name, text in sections.items()}
+    return {name: ''.join(text) for name, text in sections.items()}
 
 
 def strip_docstrings(line_gen):
@@ -119,49 +121,70 @@ def strip_docstrings(line_gen):
     return ''.join(res)
 
 
-def main(fobj, start, print=print):
-    lark_inst = Lark(fobj, parser="lalr", lexer="contextual", start=start)
+def gen_standalone(lark_inst, output=None, out=sys.stdout, compress=False):
+    if output is None:
+        output = partial(print, file=out)
 
-    print('# The file was automatically generated by Lark v%s' % lark.__version__)
-    print('__version__ = "%s"' % lark.__version__)
-    print()
+    import pickle, zlib, base64
+    def compressed_output(obj):
+        s = pickle.dumps(obj, pickle.HIGHEST_PROTOCOL)
+        c = zlib.compress(s)
+        output(repr(base64.b64encode(c)))
+
+    def output_decompress(name):
+        output('%(name)s = pickle.loads(zlib.decompress(base64.b64decode(%(name)s)))' % locals())
+
+    output('# The file was automatically generated by Lark v%s' % lark.__version__)
+    output('__version__ = "%s"' % lark.__version__)
+    output()
 
     for i, pyfile in enumerate(EXTRACT_STANDALONE_FILES):
         with open(os.path.join(_larkdir, pyfile)) as f:
             code = extract_sections(f)['standalone']
             if i:   # if not this file
                 code = strip_docstrings(partial(next, iter(code.splitlines(True))))
-            print(code)
+            output(code)
 
     data, m = lark_inst.memo_serialize([TerminalDef, Rule])
-    print( 'DATA = (' )
-    # pprint(data, width=160)
-    print(data)
-    print(')')
-    print( 'MEMO = (')
-    print(m)
-    print(')')
+    output('import pickle, zlib, base64')
+    if compress:
+        output('DATA = (')
+        compressed_output(data)
+        output(')')
+        output_decompress('DATA')
+        output('MEMO = (')
+        compressed_output(m)
+        output(')')
+        output_decompress('MEMO')
+    else:
+        output('DATA = (')
+        output(data)
+        output(')')
+        output('MEMO = (')
+        output(m)
+        output(')')
 
 
-    print('Shift = 0')
-    print('Reduce = 1')
-    print("def Lark_StandAlone(transformer=None, postlex=None):")
-    print("  return Lark._load_from_dict(DATA, MEMO, transformer=transformer, postlex=postlex)")
+    output('Shift = 0')
+    output('Reduce = 1')
+    output("def Lark_StandAlone(**kwargs):")
+    output("  return Lark._load_from_dict(DATA, MEMO, **kwargs)")
 
 
 
+
+def main():
+    make_warnings_comments()
+    parser = ArgumentParser(prog="prog='python -m lark.tools.standalone'", description="Lark Stand-alone Generator Tool",
+                            parents=[lalr_argparser], epilog='Look at the Lark documentation for more info on the options')
+    parser.add_argument('-c', '--compress', action='store_true', default=0, help="Enable compression")
+    if len(sys.argv) == 1:
+        parser.print_help(sys.stderr)
+        sys.exit(1)
+    ns = parser.parse_args()
+
+    lark_inst, out = build_lalr(ns)
+    gen_standalone(lark_inst, out=out, compress=ns.compress)
+
 if __name__ == '__main__':
-    if len(sys.argv) < 2:
-        print("Lark Stand-alone Generator Tool")
-        print("Usage: python -m lark.tools.standalone <grammar-file> [<start>]")
-        sys.exit(1)
-
-    if len(sys.argv) == 3:
-        fn, start = sys.argv[1:]
-    elif len(sys.argv) == 2:
-        fn, start = sys.argv[1], 'start'
-    else:
-        assert False, sys.argv
-
-    with codecs.open(fn, encoding='utf8') as f:
-        main(f, start)
+    main()
\ No newline at end of file
